##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################


"""

Changes: 3/10/99 Ben Leslie

Hacked around with some stuff to get file to work on my system. May be helpful
generally (I hope).


"""

__version__='$Revision: 1.5 $'[11:-2]


import string
from base64 import decodestring, encodestring
#import imaplib
import notimap
imaplib = notimap

from imapUser import imapUser

from Globals import HTMLFile, MessageDialog
from AccessControl.User import BasicUserFolder
from Acquisition import Implicit
from ExtensionClass import Base


addIMAPAdapterForm=HTMLFile('addIMAPAdapter',globals())

def addIMAPAdapter(self, title='', default_domain='',
	       smtp_server='', smtp_port=25, imap_server='',
	       imap_port=143, roles='', REQUEST=None):

    """
    Function is called by Zope to create and install a IMAPAdapter
    object.

    """
    n = IMAPAdapter(title, default_domain, smtp_server, smtp_port,
		imap_server, imap_port, roles, REQUEST)

    if hasattr(self.aq_base, 'acl_users'):
        return MessageDialog(
               title  ='Item Exists',
               message='This object already contains a User Folder',
               action ='%s/manage_main' % REQUEST['URL1'])

    self._setObject('acl_users', n)
    self.__allow_groups__=self.acl_users

    # return to the parent object's manage_main
    if REQUEST is not None: return self.manage_main(self,REQUEST)


class IMAPAdapter(BasicUserFolder, Implicit, Base):
    """ IMAPAdapter 0.0.2

    The IMAPAdapter is a user database.  It contains management hooks
    so that it can be added to a Zope folder as an 'acl_users'
    database.  It's important public method is validate() which
    returns a Zope user object of type imapUser which can be used to
    access and manage that users imap account.

    For an example of how to use the imapUser API, see the doc string
    for IMAPAdapter.imapUser.imapUser.

    """

    meta_type = 'IMAPAdapter'
    id = 'acl_users'
    title = 'IMAPAdapter'
    icon = 'misc_/IMAPAdapter/folder'

    isAUserFolder=1

    manage_options=(
        {'label':'Properties', 'action':'manage_main'},
        {'label':'Security', 'action':'manage_access'},
        )

    __ac_permissions__=(
        ('View management screens',
         ['manage', 'manage_menu', 'manage_main', 
          'manage_copyright', 'manage_tabs']),
        ('Change permissions', ['manage_access']),
        ('Change IMAPAdapter', ['manage_edit']),
        )

    manage_main=HTMLFile('properties', globals())

    def __init__(self, title='', default_domain='',
		 smtp_server='', smtp_port=25, imap_server='',
		 imap_port=143, roles='NotMail', REQUEST=None):
	""" 
        SMTP transport is currently unused.  Not a problem, cuz Zope
        comes with it's own.
        """

        self.users = {}  # this is the cache of users
        self.manage_edit(title, default_domain, smtp_server, smtp_port,
                         imap_server, imap_port, roles, REQUEST)

        
    def manage_edit(self, title, default_domain, smtp_server,
		    smtp_port, imap_server, imap_port, 
                    roles, REQUEST=None):
	""" Edit the IMAPAdapter Object
	
	"""
	self.title=title
	self.default_domain=default_domain
	self.smtp_server = smtp_server
	self.smtp_port = smtp_port
	self.imap_server = imap_server
	self.imap_port = imap_port
        self.roles = roles

        self.__roles__ = filter(None, map(string.strip, 
                                      string.split(roles, ',')))
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def validate(self, request, auth='', roles=None):
        """ """


        # if 'NotMail' not in roles:
        #    print 'Notmail not in roles'
        #    return None

        parent=request['PARENTS'][0]

        # Only do basic authentication
        if auth == None:
            return None

        if string.lower(auth[:6])!='basic ':
            return None
        name,password=tuple(string.split(decodestring(string.split(auth)[-1]), ':'))
	if self.users.has_key(name):

            try:
                # it's in the cache, check to see if it's still alive
                self.users[name].conn.connection.noop()
            except:
                # clean it out and make a new one
                del self.users[name].conn.connection
                try:
                    # cache hackery...
                    # so that #trees don't break
                    # if low level imaplib connection dies
                    c = imaplib.IMAP4(self.imap_server, self.imap_port)
                    c.login(name, password)
                except:
                    return None
                self.users[name].conn.connection = c

            return self.users[name]

        user = imapUser(name, password, self.roles, '')
        
        if not user.connect(self.imap_server, self.imap_port):
	    # no good
	    return None

	# good, cache connection
	self.users[name] = user

        # Try to authorize user
        #if user.allowed(parent, roles):
        #    return user
        #return None
        return user



